VERSION 5.00
Begin VB.Form frmDX_Record 
   BorderStyle     =   0  'None
   ClientHeight    =   1785
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   3000
   ClipControls    =   0   'False
   ControlBox      =   0   'False
   HasDC           =   0   'False
   ScaleHeight     =   1785
   ScaleWidth      =   3000
   ShowInTaskbar   =   0   'False
   Visible         =   0   'False
End
Attribute VB_Name = "frmDX_Record"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
Option Explicit

' Made by Michael Ciurescu

Private DX As New DirectX8
Private SEnum As DirectSoundEnum8
Private DIC As DirectSoundCapture8

' Buffer stuff...
Private Buff As DirectSoundCaptureBuffer8
Private BuffWave As WAVEFORMATEX
Private BuffDesc As DSCBUFFERDESC

' the buffer sizes for the DirectXSound
Private BuffLen As Long, HalfBuffLen As Long

' Events stuff...
Implements DirectXEvent8

' lets say the buffer is like this
' |---------------------------|---------------------------|
' where you see a "|" character, an event will occur
' when you read the left side of the buffer, the DirectX writes on the right side of the buffer
' and vice versa, if you read the data on the same side as DirectX writes, the sound will be messed up

Private EventsNotify() As DSBPOSITIONNOTIFY
Private EndEvent As Long, MidEvent As Long, StartEvent As Long


Public SoundVolume As Double ' if 1, ie 100%, the volume is not changed

Private memPtr As Long ' memory handle
Public ptr_WavBuffer As Long ' the pointer of the buffer

Public m_BufferLength As Long ' is the total length of the buffer

' m_BufferPos is the position where to write next in the buffer
' if this variable is not decremented regularly, the buffer will get VERY big !
Public m_BufferPos As Long

Public Event GotWavData()

Private Sub DirectXEvent8_DXCallback(ByVal eventid As Long)
    If ptr_WavBuffer = 0 Or memPtr = 0 Then
        ' if first time, then create a buffer to hold the data
        
        memPtr = GlobalAlloc(GMEM_SHARE Or GMEM_ZEROINIT, 529200)
        ptr_WavBuffer = GlobalLock(memPtr)
        
        If ptr_WavBuffer <> 0 Then m_BufferLength = 529200
        m_BufferPos = 0
    End If
    
    If m_BufferPos + HalfBuffLen >= m_BufferLength Then
        ' this should not happen, 500K is enough to store ~3 seconds of sound
        ' if it gets here, there is something wrong with the processing of the WAVE data
        
        GlobalUnlock memPtr
        
        ' make the buffer bigger...
        memPtr = GlobalReAlloc(memPtr, (m_BufferPos + (HalfBuffLen * 2)) * 1.2, GMEM_ZEROINIT)
        ptr_WavBuffer = GlobalLock(memPtr)
    End If
    
    If ptr_WavBuffer <> 0 Then
        Select Case eventid
        Case StartEvent
            Buff.ReadBuffer HalfBuffLen, HalfBuffLen, ByVal (ptr_WavBuffer + m_BufferPos), DSCBLOCK_DEFAULT
            m_BufferPos = m_BufferPos + HalfBuffLen
        Case MidEvent
            Buff.ReadBuffer 0, HalfBuffLen, ByVal (ptr_WavBuffer + m_BufferPos), DSCBLOCK_DEFAULT
            m_BufferPos = m_BufferPos + HalfBuffLen
        Case EndEvent
            Debug.Print "EndEvent"
        Case Else
            Debug.Print "EventID: " & eventid
        End Select
        
        ' let the parent control know that we have more WAVE data
        If eventid = StartEvent Or eventid = MidEvent Then
            If BuffWave.nBitsPerSample = 16 And SoundVolume <> 1 Then
                ' Change the sound volume... (digital volume)
                ChangeSoundVolumeC (ptr_WavBuffer + m_BufferPos) - HalfBuffLen, HalfBuffLen \ (BuffWave.nBitsPerSample \ 8), BuffWave.nChannels, SoundVolume, SoundVolume
            End If
            
            RaiseEvent GotWavData
        End If
    End If
End Sub

Public Function Initialize(Optional ByVal SamplesPerSec As Long = 44100, _
                            Optional ByVal BitsPerSample As Integer = 16, _
                            Optional ByVal Channels As Integer = 2, _
                            Optional ByVal HalfBufferLen As Long = 0) As String
    On Error GoTo ReturnError
    Set SEnum = DX.GetDSEnum
    Set DIC = DX.DirectSoundCaptureCreate(SEnum.GetGuid(1))
    
    BuffWave.nFormatTag = WAVE_FORMAT_PCM
    BuffWave.nChannels = Channels
    BuffWave.nBitsPerSample = BitsPerSample
    BuffWave.lSamplesPerSec = SamplesPerSec
    
    BuffWave.nBlockAlign = (BuffWave.nBitsPerSample * BuffWave.nChannels) \ 8
    BuffWave.lAvgBytesPerSec = BuffWave.lSamplesPerSec * BuffWave.nBlockAlign
    
    If HalfBufferLen <= 0 Then
        HalfBuffLen = BuffWave.lAvgBytesPerSec / 10
        HalfBuffLen = HalfBuffLen - (HalfBuffLen Mod BuffWave.nBlockAlign)
    Else
        HalfBuffLen = HalfBufferLen
    End If
    
    BuffLen = HalfBuffLen * 2
    
    BuffDesc.fxFormat = BuffWave
    BuffDesc.lBufferBytes = BuffLen
    BuffDesc.lFlags = DSCBCAPS_DEFAULT
    
    Set Buff = DIC.CreateCaptureBuffer(BuffDesc)
    
    ReDim EventsNotify(0 To 2) As DSBPOSITIONNOTIFY
    
    StartEvent = DX.CreateEvent(Me)
    EventsNotify(0).hEventNotify = StartEvent
    EventsNotify(0).lOffset = 1
    
    MidEvent = DX.CreateEvent(Me)
    EventsNotify(1).hEventNotify = MidEvent
    EventsNotify(1).lOffset = HalfBuffLen
    
    EndEvent = DX.CreateEvent(Me)
    EventsNotify(2).hEventNotify = EndEvent
    EventsNotify(2).lOffset = DSBPN_OFFSETSTOP
    
    Buff.SetNotificationPositions 3, EventsNotify()
    
    Initialize = ""
    Exit Function
ReturnError:
    Initialize = "Error: " & Err.Number & vbNewLine & _
        "Desription: " & Err.Description & vbNewLine & _
        "Source: " & Err.Source
    
    Err.Clear
    UninitializeSound
    Exit Function
End Function

Public Function SoundPlay() As Boolean
    On Error GoTo ReturnError
    
    If Not Buff Is Nothing Then Buff.Start DSCBSTART_LOOPING
    
    SoundPlay = True
    Exit Function
ReturnError:
    SoundPlay = False
    Err.Clear
End Function

Public Function SoundStop() As Boolean
    On Error GoTo ReturnError
    
    If Not Buff Is Nothing Then Buff.Stop
    
    SoundStop = True
    Exit Function
ReturnError:
    SoundStop = False
    Err.Clear
End Function

Public Sub UninitializeSound()
    On Error Resume Next
    SoundStop
    DoEvents
    
    GlobalUnlock memPtr
    GlobalFree memPtr
    memPtr = 0
    
    HalfBuffLen = 0
    
    DX.DestroyEvent EventsNotify(0).hEventNotify
    DX.DestroyEvent EventsNotify(1).hEventNotify
    DX.DestroyEvent EventsNotify(2).hEventNotify
    
    Erase EventsNotify
    
    Set Buff = Nothing
    Set DIC = Nothing
    Set SEnum = Nothing
End Sub

Private Sub Form_Initialize()
    SoundVolume = 1
End Sub

Private Sub Form_QueryUnload(Cancel As Integer, UnloadMode As Integer)
    UninitializeSound
End Sub
